local RobotBay = {}
local lg = love.graphics


function RobotBay:create( data )
	local entity = Building:create( data )
	
	entity.entityType = ENTITIES.robotBay
	
	-- also store the tile index (avoid to calculate them over and over again)
	entity.xi = data.xi
	entity.yi = data.yi
	
	-- range in full tiles
	entity.explorationRange = 4
	
	entity.image = Images["robot_bay_1"]
	entity.rendererLayer = "entities"
	
	-- selection frame image
	entity.imageWidth = 180
	entity.imageHeight = 120
	entity.outlineOffsetY = 24
	local frameImageData = Images["entity_selection_frame_1"]
	entity.selectionFrameImage = frameImageData
	entity.selectionFrameScaleX = entity.imageWidth / frameImageData.image:getWidth()
	entity.selectionFrameScaleY = entity.imageHeight / frameImageData.image:getHeight()
	
	-- blocking info in mini tiles
	-- for now each building can only occupy one full tile
	entity.blocking = {
		{0,0,0,0,0},
		{0,1,1,1,0},
		{0,1,1,0,0},
		{0,1,0,1,0},
		{0,0,0,0,0},
	}
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
		
		self.ohwSelection = OhwEntitySelection:create{
			entityId = self.id,
			width = self.imageWidth,
			height = self.imageHeight,
			offsetY = self.outlineOffsetY,
		}
		
		Game:onBuildingCreated( self )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- entity (robot) spawn location
	-- and position where they move to after being constructed
	
	-- important: position is inside the building blocking
	function entity:getTeleportUnitPosition()
		local x, y = self:getPosition()
		return x, y
	end
	
	-- important: position must not be blocked!
	function entity:getLeaveBuildingPosition()
		local x, y = self:getPosition()
		-- return x, y + 50
		return x + 50, y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set currently selected - true / false
	function entity:setSelected( selected )
		self.selected = selected
		self.animationTick = 0
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- overwrite
	-- function entity:isReadyForUpgrade()
		-- return self.remainingProductionTime == 0
	-- end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		if self.selected then
			self.animationTick = self.animationTick + dt
		end
		
		self.health:update( dt )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		
		local imgData = self.image
		local ox = imgData.ox or 0
		local oy = imgData.oy or 0
		
		lg.draw( imgData.image, x, y, 0, 1, 1, ox, oy )
		
		-- draw "light"
		-- TODO: do not draw when the building is not operating at the moment
		if true then
			local liSize = 800
			-- local alpha = 0.8
			local color = COLORS.buildingLight
			Renderer:drawCircularLight( x, y, liSize, color )
		end
		
		self.ohwSelection:setVisible( self.selected == true )
		
		-- healthbar
		-- should be about 4 to 6 times as wide as high and above the entity
		-- the width MIGHT be larger for entities with more health (?)
		if Game.drawEntityStatusOverhead or self.selected
		or self.health.timeSinceLastHealthChange < 4 then
			self.health:setVisible( true )
			local hb_y_offset = body.z + 120	-- draw above the entity
			self.health:setHealthbarPosition( x, y - hb_y_offset )
		else
			self.health:setVisible( false )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self.ohwSelection:destroy()
		self = nil
	end
	
	
	return entity
end


return RobotBay
