local RockMonster = {}
local lg = love.graphics
local pi = math.pi
local pipi = 2 * pi

-- rendered with a zoom scale of 2; size 128x128

-- TODO:
-- animation should match task / state
-- control logic seems broken: sometimes does nothing after an attack (stuck in prepareAttack)
--> because the code that was moved to onMovementDestinationReached is missing in the behavior code for prepareAttack

-- behavior:
-- they are slow walking but tough creatures
-- most often they do not spawn during the game, but are present from the start
-- rock monsters are a threat to the player base - they can damage buildings
-- when they are alone, they either sleep or walk around in a very small area
-- when disturbed, they become aggressive and attack the closest worker or building
-- until either all nearby entities are gone (too far away or dead)


function RockMonster:create( data )
	local entity = {}
	
	entity.entityType = ENTITIES.rockMonster
	entity.entityCategory = ENTITYCATEGORY.monster
	
	entity.body = {
		shape = "circle",
		x = data.x or 0,
		y = data.y or 0,
		radius = 4,
		z = 0,
	}
	entity.orientation = 0
	
	entity.maxHealth = 800
	-- entity.health = entity.maxHealth
	
	-- 25/s = one mini tile per second
	entity.movementSpeed = 40
	
	entity.task = "idle"
	
	-- world position and mini tile index
	entity.movementDestination = {}
	
	entity.rendererLayer = "entities"
	
	-- animations
	entity.animations = {}
	entity.animations.idle = Animation.new( "animations/rock monster/idle.lua" )
	entity.animations.idle:setRelativeOrigin( 0.5, 0.75 )
	entity.animations.walk = Animation.new( "animations/rock monster/idle.lua" )	-- TODO
	entity.animations.walk:setRelativeOrigin( 0.5, 0.75 )
	entity.animations.attack = Animation.new( "animations/rock monster/attack.lua" )	-- TODO: color
	entity.animations.attack:setRelativeOrigin( 0.5, 0.75 )
	
	-- set initial animation set
	entity.currentAnimation = "idle"
	
	entity.imageScale = 1
	
	-- behavior properties
	entity.wakeUpEntityDistance = 240	-- player entities can trigger this unit
	entity.attackRange = 50				-- maximum distance to the target to perform an attack
	entity.attackScanRadius = 320	-- in which attackable enemies are detected
	entity.attackDamage = 60
	entity.attackWait = 8					-- time in seconds before another attack can be started
	entity.attackPreflight = 0.8	-- time the attack animation already starts before the actual event
	entity.discardTargetDistance = 360	-- if the target is too far away, stop chasing it
	
	----------------------------------------------------------------------------------------------------
	
	function entity:load( id )
		self.id = id
		
		self.health = Health:create( self, self.maxHealth, false, 0 )	-- health component
		-- self.health:setHealthbarSize( 32, 6 )
		
		Gameloop:add( self )
		Renderer:add( self, self.rendererLayer )
		
		-- TODO: set animation callbacks
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getPosition()
		return self.body.x, self.body.y
	end
	
	
	function entity:setPosition( x, y )
		self.body.x = x
		self.body.y = y
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- get the currently possible movement speed
	function entity:getCurrentMovementSpeed()
		local speed = self.movementSpeed
		
		-- TODO: modifiers?
		
		return speed
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- internal only
	-- mini tile coordinates and world position
	function entity:_setMovementDestination( mini_xi, mini_yi, x, y )
		self.movementDestination.x = x
		self.movementDestination.y = y
		self.movementDestination.mtx = mini_xi
		self.movementDestination.mty = mini_yi
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- from extern: move to a specific location (from player move command)
	-- mini tile coordinates and world position
	function entity:moveToDestination( mini_xi, mini_yi, x, y )
		local ex, ey = self:getPosition()
		local start_txi, start_tyi = getTileIndex( ex, ey )
		local target_xi, target_yi = getTileIndex( x, y )
		
		-- extra: pre-check if there can be a path at all
		if not Pathfinding.getPath( start_txi, start_tyi, target_xi, target_yi, "ground_coarse" ) then
			return false
		end
		
		local start_xi, start_yi = getMiniTileIndex( ex, ey )
		local path = Pathfinding.getPath( start_xi, start_yi, mini_xi, mini_yi, "ground_fine" )
		if path then
			-- if canMove then
				
				-- new movement
				self:_setMovementDestination( mini_xi, mini_yi, x, y )
				self:setMovementPath( path )
				-- self:setTask( "moveTo" )
				return true
			-- end
			
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:setOrientation( angle )
		self.orientation = angle
		-- note: for some reason the angle sometimes gets larger than 2*pi instead of being negative
		if self.orientation > pipi then
			self.orientation = self.orientation - pipi
		end
		
		local currName = self.animations[self.currentAnimation]:getCurrentState()
		local animName = self:getAnimationNameByOrientation( self.orientation )
		if currName ~= animName then
			self.animations[self.currentAnimation]:setState( animName )
		end
	end
	
	
	function entity:getAnimationNameByOrientation( angle )
		local a = angle
		if a >= 0.25*pi and a < 0.75*pi then
			return "facing_sw"
		elseif a < -0.25*pi and a >= -0.75*pi then
			return "facing_ne"
		elseif math.abs( a ) >= 0.75*pi then
			return "facing_nw"
		else
			return "facing_se"
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:tick( dt )
		
		self.health:update( dt )
		
		-- behavior depending on the internal state
		self:behaviorTick( dt )
		
		
		-- animation
		if self.currentAnimation and self.animations[self.currentAnimation] then
			local anim = self.animations[self.currentAnimation]
			anim:update( dt )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- task can be:
	-- idle
	-- sleep
	-- walkAround
	-- chaseTarget
	-- prepareAttack
	-- attacking
	-- attackOver
	
	function entity:behaviorTick( dt )
		
		local ex, ey = self:getPosition()
		local random = math.random
		
		
		-- just attacked a target, now wait for attackWait
		if self._attackWait then
			self._attackWait = self._attackWait - dt
			if self._attackWait <= 0 then
				self._attackWait = nil
			end
		end
		
		
		if self.task == "idle"
		or self.task == "sleep"
		or self.task == "walkAround" then
			
			if self:arePlayerEntitiesNearby( self.wakeUpEntityDistance ) then
				self:setTask( "chaseTarget" )
			end
			
		end
		
		
		if self.task == "idle" then
			
			if random() < 0.002 then
				self:setTask( "walkAround" )
			elseif random() < 0.002 and self.health.timeSinceLastHealthChange > 10 then
				self:setTask( "sleep" )
			end
			
		elseif self.task == "sleep" then
			
			-- let the player know this mob is asleep
			-- TODO: sometimes play a snoring sound?
			
		elseif self.task == "walkAround" then
			
			if not self.movementPath then
				
				-- randomly move somewhere nearby
				local maxRange = 100
				local x = ex + random(-maxRange, maxRange)
				local y = ey + random(-maxRange, maxRange)
				local mini_xi, mini_yi = getMiniTileIndex( x, y )
				if not Game.blocking:isMiniTileBlocked( mini_xi, mini_yi ) then
					self:moveToDestination( mini_xi, mini_yi, x, y )
				end
				-- in case the targeted position is blocked, just wait and try again in the next tick
				
			end
			
		elseif self.task == "attacking" then
			
			local target = EntityManager:getEntityById( self.targetId )
			if target then
				
				self._attackPreflight = self._attackPreflight - dt
				if self._attackPreflight <= 0 then
					self._attackPreflight = nil
					
					-- finally damage the target
					self:attackTarget( target )
					
					self:setTask( "attackOver" )
				end
				
			else
				-- cancel if the target is not available anymore
				self:setTarget( nil )
				self:setTask( "idle" )
			end
			
		elseif self.task == "prepareAttack" then
			
			local target = EntityManager:getEntityById( self.targetId )
			if not target
			or Tools.getDistance2( ex, ey, target:getPosition() ) > self.discardTargetDistance then
				
				self:setTarget( nil )
				self:setTask( "idle" )
				
			else
				
				-- attack as soon as the target is in the attackRange
				local target_x, target_y = target:getPosition()
				local mini_xi, mini_yi = getMiniTileIndex( target_x, target_y )
				local destination = self.movementDestination
				if destination and (destination.mtx ~= mini_xi or destination.mty ~= mini_yi) then
					-- print( "rock monster: target moved" )
					self:moveToDestination( mini_xi, mini_yi, target_x, target_y )
				end
				local dist = Tools.getDistance2( ex, ey, target_x, target_y )
				if dist <= self.attackRange and not self._attackWait then
					self:startAttack( target )
				end
				
			end
			
		elseif self.task == "attackOver" then
			
			self:setTask( "idle" )
			
		elseif self.task == "chaseTarget" then
			
			-- if targeting some entity, go after it
			--> attack when in range, discard when too far away
			
			-- no target yet? look for one
			local targets = self:getAttackableTargets()
			local num_targets = #targets
			if num_targets > 0 then
				print( "rock monster found", num_targets, "targets" )
				local i = random( 1, num_targets )
				self:setTarget( targets[i] )
			end
			
		end
		
		
		-- currently want to move somewhere
		if self.movementPath then
			local nSteps = #self.movementPath
			if nSteps > 0 then
				local start_xi, start_yi = getMiniTileIndex( ex, ey )
				if start_xi == self.movementPath[nSteps].x
				and start_yi == self.movementPath[nSteps].y then
					table.remove( self.movementPath, nSteps )
				else
					-- move to the next mini tile center
					local goal_x, goal_y = miniTileIndexToPosition( self.movementPath[nSteps].x, self.movementPath[nSteps].y )
					local direction = Tools.vectorSub( self.body, { x = goal_x, y = goal_y } )
					local vec2 = Tools.vectorSetLength( direction, self:getCurrentMovementSpeed() )
					self:move( vec2.x * dt, vec2.y * dt )
				end
			else
				-- already on the final mini tile
				-- move to the exact target location
				local direction = Tools.vectorSub( self.body, self.movementDestination )
				local vec2 = Tools.vectorTrim( direction, self:getCurrentMovementSpeed() * dt )
				self:move( vec2.x, vec2.y )
				
				if math.abs( self.body.x - self.movementDestination.x ) < 2
				and math.abs( self.body.y - self.movementDestination.y ) < 2 then
					self.movementPath = nil
					self:onMovementDestinationReached()
				end
			end
		end
			
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:startAttack( target )
		if self._attackWait then
			print( "rock monster:startAttack - must wait for _attackWait:", self._attackWait )
			return false
		end
		self:stopMovement()
		self:setTask( "attacking" )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:attackTarget( target )
		print( "rock monster:ATTACKTARGET", target.id )
		Game:onEntityHurtEntity( self, target, self.attackDamage )
		-- TODO: sound
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- entity table
	function entity:setTarget( target )
		if not target then
			self.targetId = nil
			self:setTask( "idle" )
			return false
		end
		
		local x, y = target:getPosition()
		local mini_xi, mini_yi = getMiniTileIndex( x, y )
		if not self:moveToDestination( mini_xi, mini_yi, x, y ) then
			self.targetId = nil
			self:setTask( "idle" )
			return false
		end
		print( "rock monster:setTarget", target.id )
		self.targetId = target.id
		self:setTask( "prepareAttack" )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- immediately stop the entity
	function entity:stopMovement()
		self.movementPath = nil
		self.movementDestination = {}
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:onMovementDestinationReached()
		
		local ex, ey = self:getPosition()
		local random = math.random
		
		-- TODO
		if self.task == "walkAround" then
			self:setTask( "idle" )
			
		elseif self.task == "prepareAttack" then
			
			-- local target = EntityManager:getEntityById( self.targetId )
			-- if target then
				-- local target_x, target_y = target:getPosition()
				-- local mini_xi, mini_yi = getMiniTileIndex( target_x, target_y )
				-- local destination = self.movementDestination
				-- if destination and (destination.mtx ~= mini_xi or destination.mty ~= mini_yi) then
					-- print( "rock monster: target moved" )
					-- self:moveToDestination( mini_xi, mini_yi, target_x, target_y )
				-- end
			-- end
			
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:setMovementPath( path )
		self.movementPath = path
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:move( dx, dy )
		self.body.x = self.body.x + (dx or 0)
		self.body.y = self.body.y + (dy or 0)
		self:setOrientation( Tools.getAngleFromVector{ x = dx, y = dy } )
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- set animation etc.
	function entity:setTask( task )
		
		-- already doing that?
		if self.task == task then
			return
		end
		
		-- update task
		self.task = task
		print( "rock monster setTask:", task )
		
		if task == "idle" then
			self.currentAnimation = "idle"
		
		elseif task == "prepareAttack" then
			
		elseif task == "attacking" then
			
			self.currentAnimation = "attack"
			
			-- start the animation a moment before the actual attack happens
			self._attackPreflight = self.attackPreflight
			
		elseif task == "attackOver" then
			
			self._attackWait = self.attackWait
			
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:arePlayerEntitiesNearby( radius )
		local x, y = self:getPosition()
		local targets = EntityManager:getEntitiesFilter( function(e)
			if e.isPlayerControlled then
				local ex, ey = e:getPosition()
				return Tools.getDistance2( x, y, ex, ey ) <= radius
			end
		end )
		return #targets > 0
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:getAttackableTargets()
		local x, y = self:getPosition()
		local targets = EntityManager:getEntitiesFilter( function(e)
			if e.entityType == ENTITIES.rockRaider and not e:isDrivingVehicle()
			-- TODO: do not attack construction site and fence
			-- or e.entityCategory == ENTITYCATEGORY.building
			or e.entityCategory == ENTITYCATEGORY.vehicle then
				local ex, ey = e:getPosition()
				return Tools.getDistance2( x, y, ex, ey ) <= self.attackScanRadius
			end
		end )
		return targets
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- called from game when a hurt event occured
	function entity:onHurt( damage )
		
		-- TODO: on hurt sound
		
	end
	
	
	function entity:onHealthChanged()
		if self.health:getHealth() <= 0 then
			self:die()
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:die()
		-- TODO: animation, sound ...
		EntityManager:remove( self )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:draw()
		
		local body = self.body
		local x, y = convertToDimetric( body.x, body.y )
		
		-- animation
		if self.currentAnimation and self.animations[self.currentAnimation] then
			local anim = self.animations[self.currentAnimation]
			anim:setPosition( x, y )
			anim:draw( self.imageScale, self.imageScale )
		end
		
		lg.circle( "line", x, y, 6 )	-- debug
		
		-- healthbar
		if Game.drawEntityStatusOverhead
		or self.health.timeSinceLastHealthChange < 4 then
			self.health:setVisible( true )
			local hb_y_offset = body.z + 48	-- draw above the entity
			self.health:setHealthbarPosition( x, y - hb_y_offset )
		else
			self.health:setVisible( false )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function entity:destroy()
		Gameloop:remove( self )
		Renderer:remove( self, self.rendererLayer )
		self.health:remove()
		self = nil
	end
	
	
	return entity
end


return RockMonster
