local Mission = {}

Mission.displayName = "Recovery"

Mission.mapName = "testmap3"

Mission.description = getString( "mission_description_3" )


function Mission.onGameStart()
	print( "mission 3 started" )
	
	-- landslides occur more often than usual
	Game:setLandslideChance( 0.01 )
	
	-- create a tool store and set the camera
	local toolStore = Game:spawnBuilding( ENTITIES.toolStore, 9, 26 )
	local x, y = toolStore:getPosition()
	Camera:setPosition( x, y )
	
	-- create a teleport pad
	Game:spawnBuilding( ENTITIES.teleportPad, 9, 27 )
	
	-- where to bring the vehicle
	Mission.rescuePositionXi = 9
	Mission.rescuePositionYi = 26
	-- vehicle must be near the target tile
	Mission.rescueTileDistance = 2
	
	-- the vehicle to retrieve
	do
		local x, y = tileIndexToPosition( 14, 3 )
		local vehicle = EntityManager:createEntity(
			ENTITIES.hoverScout,
			{ x = x, y = y }
		)
		
		Mission.rescueVehicleId = vehicle.id
		print( "rescue entity ID:", vehicle.id )
		
		-- this entity should not be player controlled for now
		vehicle:setIsPlayerControlled( false )
		
	end
	
	-- victory condition
	Game:trackJob( Simplejob:addJob{
		intervall = 1,
		callback = function()
			if GUImgr:getCurrentScreenId() ~= Game.id then
				return true
			end
			if Game.isPaused then return end
			
			if Mission.checkVictoryCondition() then
				Game:victory()
				return true
			end
		end,
	} )
	
end


function Mission.checkVictoryCondition()
	local vehicle = EntityManager:getEntityById( Mission.rescueVehicleId )
	if not vehicle then
		-- TODO: mission failed
		print( "mission: vehicle is gone" )
		return false
	end
	
	local x, y = vehicle:getPosition()
	local xi, yi = getTileIndex( x, y )
	local dx = math.abs( Mission.rescuePositionXi - xi )
	local dy = math.abs( Mission.rescuePositionYi - yi )
	return dx+dy <= Mission.rescueTileDistance
end


return Mission
