local CustomMissionScreen = {}
local screenId = "custom_mission"

--[[
options that require UI elements:
- generate new map
- map preview
- map size
- water yes/no/only rivers/only sea
- lava yes/no
- cave depth - normal/icy/??? environment
- monster (spawn, amount, frequency, types)
- oxygen yes/no (slow/fast)
- energy crystal amount
- environmental parameters (lava spread speed, landslide chance)
- mission objective (main + secondary)
- Rock Raider limit
- tech limits?
- start cave size + start buildings
- level seed + option to re-use one if the player likes a map
--]]

function CustomMissionScreen:create()
	local screen = {}
	
	screen.z = 1
	screen.id = screenId
	screen.screenId = screenId
	screen.bgColor = { 0.1, 0.1, 0.2 }
	
	
	function screen:open()
		GUImgr:setScreen( self.id )
		Logger:log( "Open custom mission menu." )
	end
	
	
	function screen:close()
		Logger:log( "Close custom mission menu." )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function screen:createNewMap()
		
		-- options for map generator
		local options = {}
		
		options.seed = nil	-- TODO: can fetch from text input element?
		-- note: different parameters in the options will result in different maps even with the same seed
		
		-- fetch map size from some GUI element
		local sizeText, index = GUImgr:getObjectById( "btnCustomMissionMapSize" ):getSelectedItem()
		-- should be multiples of 8
		local sizes = { 16, 24, 32 }
		assert( sizes[index] )
		local size = sizes[index]
		
		-- energy crystal amount
		local _, index = GUImgr:getObjectById( "btnCustomMissionEnergyCrystalAmount" ):getSelectedItem()
		local ec = { "few", "normal", "plenty" }
		assert( sizes[index] )
		options.energyCrystalAmount = ec[index]
		
		local _, index = GUImgr:getObjectById( "btnCustomMissionWaterOptions" ):getSelectedItem()
		-- 0 = no water
		-- 1 = rivers only
		-- 2 = lakes only
		-- 3 = rivers+lakes
		options.water = index
		
		-- can define various parameters in options table (water, resources, monsters etc.)
		local map, script = MapGenerator:createMap( size, size, options )
		
		self.map = map
		self.script = script
		
		-- update the preview
		local mapPreview = GUImgr:getObjectById( "mapCustomMissionMenuMapPreview" )
		mapPreview:updateMap( map )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- CREATE A NEW MAP
	Button:create{
		id = "btnCustomMissionCreateMap",
		x = SCREEN.width * 0.8,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "custom_mission_create_map" ),
		fSize = 22,
		callback = function()
			screen:createNewMap()
		end,
		screenId = screenId,
		-- enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECT MAP SIZE
	SwitchButton:create{
		id = "btnCustomMissionMapSize",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.2 - 2,
		sizeY = SCREEN.height * 0.1 - 2,
		items = {
			getString( "custom_mission_map_size_1" ),
			getString( "custom_mission_map_size_2" ),
			getString( "custom_mission_map_size_3" ),
		},
		itemIndex = 2,	-- initial index
		fSize = 22,
		callback = function()
			-- print( "select map size" )
		end,
		screenId = screenId,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- AMOUNT OF ENERGY CRYSTALS
	SwitchButton:create{
		id = "btnCustomMissionEnergyCrystalAmount",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2 - 2,
		sizeY = SCREEN.height * 0.1 - 2,
		items = {
			getString( "custom_mission_energy_crystals_1" ),
			getString( "custom_mission_energy_crystals_2" ),
			getString( "custom_mission_energy_crystals_3" ),
		},
		-- itemIndex = 1,	-- initial index
		fSize = 22,
		callback = function()
		end,
		screenId = screenId,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- WATER OPTIONS
	SwitchButton:create{
		id = "btnCustomMissionWaterOptions",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.2 - 2,
		sizeY = SCREEN.height * 0.1 - 2,
		items = {
			getString( "custom_mission_water_1" ),	-- none
			getString( "custom_mission_water_2" ),	-- river
			getString( "custom_mission_water_3" ),	-- lake
			getString( "custom_mission_water_4" ),	-- river+lake
		},
		itemIndex = 3,	-- initial index
		fSize = 22,
		callback = function()
		end,
		screenId = screenId,
		-- enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- LAVA OPTIONS
	SwitchButton:create{
		id = "btnCustomMissionLavaOptions",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.7,
		sizeX = SCREEN.width * 0.2 - 2,
		sizeY = SCREEN.height * 0.1 - 2,
		items = {
			getString( "custom_mission_lava_1" ),
			getString( "custom_mission_lava_2" ),
			getString( "custom_mission_lava_3" ),
			getString( "custom_mission_lava_4" ),
		},
		-- itemIndex = 1,	-- initial index
		fSize = 22,
		callback = function()
		end,
		screenId = screenId,
		enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MAP PREVIEW
	do
		-- local size = math.min( SCREEN.width * 0.2, SCREEN.height * 0.4 )
		MapPreview:create{
			id = "mapCustomMissionMenuMapPreview",
			x = SCREEN.width * 0.8,
			y = SCREEN.height * 0.65,
			sizeX = SCREEN.width * 0.2,
			sizeY = SCREEN.height * 0.4,
			screenId = screenId,
		}
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- START MISSION
	Button:create{
		id = "btnCustomMissionMenuStartMap",
		x = SCREEN.width * 0.8,
		y = SCREEN.height * 0.9,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "custom_mission_menu_start_map" ),
		fSize = 22,
		callback = function()
			if not screen.map or not screen.script then
				return false
			end
			
			screen:close()
			GameScreen:open()	-- game:open is called from the screen
			Game:startCustomMission( screen.map, screen.script )
			return true
		end,
		screenId = screenId,
		-- enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- QUIT TO MAIN MENU
	Button:create{
		id = "btnCustomMissionMenuQuit",
		x = SCREEN.width * 0.2,
		y = SCREEN.height * 0.9,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "custom_mission_menu_menu_quit" ),
		fSize = 22,
		callback = function()
			screen:close()
			MainMenu:open()
		end,
		screenId = screenId,
	}
	
	----------------------------------------------------------------------------------------------------
	
	Mainloop:add( screen )
	GUImgr:addScreen( screen )
	-- UIRenderer:add( screen )
	
	return screen
end


return CustomMissionScreen
