-- GUI elements for the game screen
-- GUI elements for the game screen
local GameScreen = {}


function GameScreen:create()
	local game = {}
	
	game.id = "game"
	
	function game:open()
		GUImgr:setScreen( self.id )
		
		Music:switchSetToUse( "game" )
		
		Game:open()
	end
	
	----------------------------------------------------------------------------------------------------
	
	function game:switchToSelectionState( newState )
		
		-- optional: perform something to clean up the old state?
		GUImgr:setVisibility( "widGameGlobalSelection", false )
		GUImgr:setVisibility( "widGameWallSelection", false )
		GUImgr:setVisibility( "widGameBuildingSelection", false )
		GUImgr:setVisibility( "widGameEntitySelection", false )
		GUImgr:setVisibility( "widGameTileSelection", false )
		
		-- and show and hide the correct widgets and buttons...
		if newState == "global" then
			GUImgr:setVisibility( "widGameGlobalSelection", true )
			
		elseif newState == "wall" then
			GUImgr:setVisibility( "widGameWallSelection", true )
			
		elseif newState == "building" then
			GUImgr:setVisibility( "widGameBuildingSelection", true )
			
		elseif newState == "entity" then
			GUImgr:setVisibility( "widGameEntitySelection", true )
			
		elseif newState == "tile" then
			GUImgr:setVisibility( "widGameTileSelection", true )
			
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- GLOBAL SELECTION STATE = NO ENTITY IS CURRENTLY SELECTED
	Widget:create{
		id = "widGameGlobalSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- WALL SELECTION
	Widget:create{
		id = "widGameWallSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- BUILDING SELECTION
	Widget:create{
		id = "widGameBuildingSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- ENTITY SELECTION
	Widget:create{
		id = "widGameEntitySelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- (EMPTY OR WITH DEBRIS?) TILE SELECTION
	Widget:create{
		id = "widGameTileSelection",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- BUILDIG MENU (select new building to create at selected tile)
	Widget:create{
		id = "widGameBuildingMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.8,
		screenId = "game",
		visible = false,
	}
	
	-- NEW VEHICLE MENU (opened from vehicle factory)
	Widget:create{
		id = "widGameVehicleFactoryNewVehicleMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- NEW ROBOT MENU (opened from robot bay)
	Widget:create{
		id = "widGameRobotBayNewRobotMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		visible = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECTED WALL
	
	-- MINE IT
	Button:create{
		id = "btnGameActionDrillWall",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_drill_wall"],
		callback = function()
			Game:guiActionDrillWall()
		end,
		screenId = "game",
		parentId = "widGameWallSelection",
		tooltip = { text = getString( "game_action_drill_wall_tooltip" ) },
	}
	
	-- REINFORCE IT
	Button:create{
		id = "btnGameActionReinforceWall",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_reinforce_wall"],
		callback = function()
			Game:guiActionReinforceWall()
		end,
		screenId = "game",
		parentId = "widGameWallSelection",
		tooltip = { text = getString( "game_action_reinforce_wall_tooltip" ) },
	}
	
	-- CANCEL DRILL WALL
	Button:create{
		id = "btnGameActionCancelDrillWall",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_no"],	-- TODO
		callback = function()
			Game:guiActionCancelDrillWall()
		end,
		screenId = "game",
		parentId = "widGameWallSelection",
		tooltip = { text = getString( "game_action_cancel_drill_wall_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECTED TILE
	
	-- REMOVE DEBRIS
	Button:create{
		id = "btnGameActionRemoveDebris",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_clear_debris"],
		callback = function()
			
			-- debris (tile) selected? -> add a task to a job queue
			local debris = Game.selectedEntity
			if not debris
			or debris.entityType ~= ENTITIES.debris then
				return false
			end
			
			-- DEBUG
			if love.keyboard.isDown( "lshift" ) then
				-- game:debugDestroySelectedDebris()
				debris:removeMass()
				-- game:reselectCurrentTile()
				return
			end
			-- /DEBUG
			
			Game:addClearDebrisJob( debris )
			
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_action_remove_debris_tooltip" ) },
		enabled = false,
	}
	
	-- CREATE BUILDING AREA
	Button:create{
		id = "btnGameActionCreateBuildingArea",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_building_area"],
		callback = function()
			if love.keyboard.isDown( "lshift" ) then
				Game:debugCreateBuildingArea()
				return
			end
			Game:guiActionCreateBuildingArea()
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_action_create_building_area_tooltip" ) },
	}
	
	-- OPEN BUILDING MENU
	Button:create{
		id = "btnGameToggleBuildingMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Gebäude bauen",
		imageData = Images["icon_construct_building"],
		-- fSize = 22,
		callback = function()
			if Game.isPaused then return end
			GUImgr:toggleVisibility( "widGameBuildingMenu" )
			-- hide conflicting widgets
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_building_menu_tooltip" ) },
		enabled = false,
	}
	
	-- REPAIR EROSION DAMAGE
	Button:create{
		id = "btnGameActionRepairErosionDamage",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_repair_erosion_damage"],
		-- fSize = 22,
		callback = function()
			-- if Game.isPaused then return end
			
			if not Game.selectedTile then
				return false
			end
			
			local xi, yi = Game.selectedTile.xi, Game.selectedTile.yi
			Game:addRepairErosionDamageJob( xi, yi )
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_action_repair_erosion_damage_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- building buttons
	do
		
		-- see position and size of widGameBuildingMenu!
		local buttonSize = Tools.min( SCREEN.width * 0.09, SCREEN.height * 0.09 )
		local x = SCREEN.width * 0.85
		local top = SCREEN.height * 0.15
		local buildingButtons = {
			{ id = "btnGameActionBuildToolStore", icon = Images["icon_tool_store"],
				entityType = ENTITIES.toolStore, tooltipKey = "game_build_tool_store_tooltip" },
			{ id = "btnGameActionBuildTeleportPad", icon = Images["icon_teleport_pad"],
				entityType = ENTITIES.teleportPad, tooltipKey = "game_build_teleport_pad_tooltip" },
			{ id = "btnGameActionBuildPowerStation", icon = Images["icon_power_station"],
				entityType = ENTITIES.powerStation, tooltipKey = "game_build_power_station_tooltip" },
			{ id = "btnGameActionBuildSupportStation", icon = Images["icon_support_station"],
				entityType = ENTITIES.supportStation, tooltipKey = "game_build_support_station_tooltip" },
			-- { id = "btnGameActionBuildResearchCenter", icon = Images["icon_construct_building"], entityType = nil, },	-- TODO
			{ id = "btnGameActionBuildVehicleFactory", icon = Images["icon_vehicle_factory"],
				entityType = ENTITIES.vehicleFactory, tooltipKey = "game_build_vehicle_factory_tooltip" },
			{ id = "btnGameActionBuildRobotBay", icon = Images["icon_robot_bay"],
				entityType = ENTITIES.robotBay, tooltipKey = "game_build_robot_bay_tooltip" },
			{ id = "btnGameActionBuildTeslaTower", icon = Images["icon_tesla_tower"],
				entityType = ENTITIES.teslaTower, tooltipKey = "game_build_tesla_tower_tooltip" },
			{ id = "btnGameActionBuildWaterGunTower", icon = Images["icon_water_gun_tower"],
				entityType = ENTITIES.waterGunTower, tooltipKey = "game_build_water_gun_tower_tooltip" },
		}
		for i, btn in ipairs( buildingButtons ) do
			local x = x
			local y = top + (i-1) * SCREEN.height * 0.1
			Button:create{
				id = btn.id,
				x = x,
				y = y,
				sizeX = buttonSize,
				sizeY = buttonSize,
				imageData = btn.icon,
				callback = function()
					Game:guiActionCreateBuilding( btn.entityType )
				end,
				screenId = "game",
				parentId = "widGameBuildingMenu",
				tooltip = {
					text = getString( btn.tooltipKey ),
				},
			}
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- VEHICLE FACTORY: TOGGLE NEW VEHICLE MENU
	Button:create{
		id = "btnGameVehicleFactoryToggleNewVehicleMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Fahrzeuge",
		-- fSize = 22,
		imageData = Images["icon_vehicle"],
		callback = function()
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
			GUImgr:toggleVisibility( "widGameVehicleFactoryNewVehicleMenu" )
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		enabled = true,
		tooltip = { text = getString( "game_toggle_vehicle_menu_tooltip" ) },
	}
	
	-- new vehicle buttons
	do
		local buttonSize = Tools.min( SCREEN.width * 0.09, SCREEN.height * 0.09 )
		local x = SCREEN.width * 0.85
		local top = SCREEN.height * 0.15
		local buttons = {
			{ id = "btnGameActionCreateHoverScout", icon = Images["icon_hover_scout"],
				entityType = ENTITIES.hoverScout, tooltipKey = "game_create_hover_scout_tooltip", },
			{ id = "btnGameActionCreateDigger", icon = Images["icon_digger"],
				entityType = ENTITIES.digger, tooltipKey = "game_create_digger_tooltip", },
			{ id = "btnGameActionCreateTransportTruck", icon = Images["icon_transport_truck"],
				entityType = ENTITIES.transportTruck, tooltipKey = "game_create_transport_truck_tooltip", },
			{ id = "btnGameActionCreateLoaderDozer", icon = Images["icon_loader_dozer"],
				entityType = ENTITIES.loaderDozer, tooltipKey = "game_create_loader_dozer_tooltip", },
			{ id = "btnGameActionCreateMobileLaserCutter", icon = Images["icon_mobile_laser_cutter"],
				entityType = ENTITIES.mobileLaserCutter, tooltipKey = "game_create_mobile_laser_cutter_tooltip", },
		}
		
		for i, btn in ipairs( buttons ) do
			Button:create{
				id = btn.id,
				x = x,
				y = top + (i-1) * SCREEN.height * 0.1,
				shape = "rect",
				sizeX = buttonSize,
				sizeY = buttonSize,
				imageData = btn.icon,
				callback = function()
					Game:guiActionCreateVehicle( btn.entityType )
				end,
				screenId = "game",
				parentId = "widGameVehicleFactoryNewVehicleMenu",
				tooltip = {
					text = getString( btn.tooltipKey ),
				},
			}
		end
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- ROBOT BAY: TOGGLE NEW ROBOT MENU
	Button:create{
		id = "btnGameRobotBayToggleNewRobotMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_probe_droid"],	-- TODO
		callback = function()
			-- GUImgr:setVisibility( "widGameWorkerJobPriorities", false )
			GUImgr:toggleVisibility( "widGameRobotBayNewRobotMenu" )
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		tooltip = { text = getString( "game_toggle_robot_menu_tooltip" ) },
	}
	
	-- NEW ROBOT: PROBE DROID
	Button:create{
		id = "btnGameActionCreateProbeDroid",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_probe_droid"],
		callback = function()
			Game:guiActionCreateRobot( ENTITIES.probeDroid )
		end,
		screenId = "game",
		parentId = "widGameRobotBayNewRobotMenu",
		tooltip = {
			text = getString( "game_create_probe_droid_tooltip" ),
		},
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- UPGRADE BUILDING
	Button:create{
		id = "btnGameActionUpgradeBuilding",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		callback = function()
			Game:guiActionUpgradeBuilding()
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		imageData = Images["icon_upgrade_building"],
		tooltip = {
			text = getString( "game_upgrade_building_tooltip" ),
		},
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- REPAIR BUILDING
	Button:create{
		id = "btnGameActionRepairBuilding",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		callback = function()
			Game:guiActionRepairBuilding()
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		imageData = Images["icon_repair_building"],
		tooltip = { text = getString( "game_action_repair_building_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- TELEPORT PAD: TELEPORT RR DOWN
	Button:create{
		id = "btnGameActionTeleportRockRaiderToBuilding",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Rock Raider hierher teleportieren",
		-- fSize = 22,
		callback = function()
			Game:guiActionTeleportRockRaiderToBuilding()
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		imageData = Images["icon_rock_raider_1"],
		tooltip = {
			text = getString( "game_teleport_rock_raider_to_building_tooltip" ),
			-- enabled = true,
			-- duration = -1,
		},
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- ROCK RAIDER, VEHICLE ETC.: EXPELL/TELEPORT UP
	Button:create{
		id = "btnGameActionExpellUnit",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_expell_unit" ),
		-- fSize = 22,
		imageData = Images["icon_expell_unit"],
		callback = function()
			Game:guiActionExpellUnit()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = {
			text = getString( "game_action_expell_unit_tooltip" ),
		},
	}
	
	-- DROP ALL CARRIED ITEMS
	-- Button:create{
		-- id = "btnGameActionEntityDropCarriedEntities",
		-- x = SCREEN.width * 0.95,
		-- y = SCREEN.height * 0.25,
		-- sizeX = SCREEN.width * 0.09,
		-- sizeY = SCREEN.height * 0.09,
		-- autoSizeSquare = true,
		-- imageData = Images["icon_arrow_down"],
		-- callback = function()
			-- Game:guiActionEntityDropCarriedEntities()
		-- end,
		-- screenId = "game",
		-- parentId = "widGameEntitySelection",
		-- tooltip = {
			-- text = getString( "game_action_drop_entities_tooltip" ),
		-- },
		-- visible = false,
	-- }
	
	-- CANCEL THE CURRENT JOB
	Button:create{
		id = "btnGameActionEntityCancelCurrentJob",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_no"],
		callback = function()
			Game:guiActionEntityCancelCurrentJob()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		tooltip = {
			text = getString( "game_action_cancel_current_job_tooltip" ),
		},
		visible = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- ROCK RAIDER OPERATION MODE (automatic job assignment yes/no, alarm mode)
	Widget:create{
		id = "widGameRockRaiderOperationModeMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.3,
		screenId = "game",
		visible = false,
	}
	
	-- ROCK RAIDER: TOGGLE WIDGET FOR OPERATION MODE
	Button:create{
		id = "btnGameToggleRockRaiderOpModeMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_settings"],
		callback = function()
			GUImgr:toggleVisibility( "widGameRockRaiderOperationModeMenu" )
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		tooltip = {
			text = getString( "game_rock_raider_select_op_mode_tooltip" ),
		},
		visible = false,
	}
	
	-- ROCK RAIDER: ACTIVATE AUTOMATIC JOB ASSIGNMENT
	Button:create{
		id = "btnGameActionRockRaiderActivateAutomaticJobAssignment",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_gear_wheel_yes"],
		callback = function()
			Game:guiActionSetRockRaiderOpMode( 1 )
		end,
		screenId = "game",
		parentId = "widGameRockRaiderOperationModeMenu",
		tooltip = {
			text = getString( "game_rock_raider_activate_automatic_job_assignment_tooltip" ),
		},
	}
	
	-- ROCK RAIDER: DEACTIVATE AUTOMATIC JOB ASSIGNMENT
	Button:create{
		id = "btnGameActionRockRaiderDeactivateAutomaticJobAssignment",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_gear_wheel_no"],
		callback = function()
			Game:guiActionSetRockRaiderOpMode( 2 )
		end,
		screenId = "game",
		parentId = "widGameRockRaiderOperationModeMenu",
		tooltip = {
			text = getString( "game_rock_raider_deactivate_automatic_job_assignment_tooltip" ),
		},
	}
	
	-- ROCK RAIDER: ALARM (COMBAT) MODE
	Button:create{
		id = "btnGameActionRockRaiderActivateAlarmMode",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.55,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_settings"],	-- TODO
		callback = function()
			Game:guiActionSetRockRaiderOpMode( 3 )
		end,
		screenId = "game",
		parentId = "widGameRockRaiderOperationModeMenu",
		tooltip = {
			text = getString( "game_rock_raider_enter_alarm_mode_tooltip" ),
		},
		enabled = false,	-- mode is not yet implemented
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- DESTROY SELECTED BUILDING
	Button:create{
		id = "btnGameActionDestroyBuilding",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_destroy_building"],
		callback = function()
			Game:guiActionDestroyBuilding()
		end,
		screenId = "game",
		parentId = "widGameBuildingSelection",
		tooltip = { text = getString( "game_action_destroy_building_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- DESTROY SELECTED BUILDING AREA
	Button:create{
		id = "btnGameActionDestroyBuildingArea",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_destroy_building"],
		callback = function()
			Game:guiActionDestroyBuildingArea()
		end,
		screenId = "game",
		parentId = "widGameTileSelection",
		tooltip = { text = getString( "game_action_destroy_building_area_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- VEHICLE: GET DRIVER
	Button:create{
		id = "btnGameActionVehicleGetDriver",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_vehicle_get_driver"],
		callback = function()
			Game:guiActionVehicleGetDriver()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = { text = getString( "game_vehicle_get_driver_tooltip" ) },
	}
	
	-- VEHICLE: DRIVER LEAVE
	Button:create{
		id = "btnGameActionVehicleDriverLeave",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_vehicle_driver_leave" ),
		imageData = Images["icon_vehicle_driver_leave"],
		callback = function()
			Game:guiActionVehicleDriverLeave()
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = { text = getString( "game_vehicle_driver_leave_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- VEHICLE / ROBOT: UPGRADE ENTITY MENU
	Widget:create{
		id = "widGameUpgradeEntityMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.5,
		screenId = "game",
		visible = false,
	}
	
	-- TOGGLE UPGRADE ENTITY MENU
	Button:create{
		id = "btnGameToggleUpgradeEntityMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_vehicle_driver_leave" ),
		imageData = Images["icon_upgrade_building"],
		callback = function()
			GUImgr:toggleVisibility( "widGameUpgradeEntityMenu" )
		end,
		screenId = "game",
		parentId = "widGameEntitySelection",
		visible = false,
		tooltip = { text = getString( "game_upgrade_entity_menu_tooltip" ) },
	}
	
	-- ENTITY UPGRADE: DRILL EFFICIENCY
	Button:create{
		id = "btnGameActionEntityUpgradeDrillEfficiency",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.15,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_upgrade_building"],
		callback = function()
			Game:guiActionUpgradeEntity( "drill_efficiency" )
		end,
		screenId = "game",
		parentId = "widGameUpgradeEntityMenu",
		visible = false,
		tooltip = { text = getString( "game_upgrade_entity_drill_efficiency_tooltip" ) },
	}
	
	-- ENTITY UPGRADE: MOVEMENT SPEED
	Button:create{
		id = "btnGameActionEntityUpgradeMovementSpeed",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_upgrade_building"],
		callback = function()
			Game:guiActionUpgradeEntity( "movement_speed" )
		end,
		screenId = "game",
		parentId = "widGameUpgradeEntityMenu",
		visible = false,
		tooltip = { text = getString( "game_upgrade_entity_movement_speed_tooltip" ) },
	}
	
	-- ENTITY UPGRADE: CARGO CAPACITY
	Button:create{
		id = "btnGameActionEntityUpgradeCargoCapacity",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_upgrade_building"],
		callback = function()
			Game:guiActionUpgradeEntity( "cargo_capacity" )
		end,
		screenId = "game",
		parentId = "widGameUpgradeEntityMenu",
		visible = false,
		tooltip = { text = getString( "game_upgrade_entity_cargo_capacity_tooltip" ) },
	}
	
	-- ENTITY UPGRADE: LAVA SHIELD
	
	-- ENTITY UPGRADE: ?
	
	----------------------------------------------------------------------------------------------------
	
	Widget:create{
		id = "widGameResourceOverview",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		visible = true,
	}
	
	local btnEC = Button:create{
		id = "btnGameResourceAmountEnergyCrystal",
		x = SCREEN.width * 0.35,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["energy_crystal"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_energy_crystal_tooltip" ) },
	}
	btnEC:setTextColors( COLORS.white, COLORS.white, COLORS.white, COLORS.white )
	
	local btnOre = Button:create{
		id = "btnGameResourceAmountOre",
		x = SCREEN.width * 0.45,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["ore_normal"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_ore_tooltip" ) },
	}
	btnOre:setTextColors( COLORS.white, COLORS.white, COLORS.white, COLORS.white )
	
	local btnRS = Button:create{
		id = "btnGameResourceAmountRockSample",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.height * 0.08,
		sizeY = SCREEN.height * 0.08,
		text = "0",
		fSize = 22,
		imageData = Images["rock_sample_1"],
		screenId = "game",
		parentId = "widGameResourceOverview",
		tooltip = { text = getString( "game_stored_rock_sample_tooltip" ) },
	}
	btnRS:setTextColors( COLORS.white, COLORS.white, COLORS.white, COLORS.white )
	
	----------------------------------------------------------------------------------------------------
	
	-- TODO: text alignment in both directions
	local btnJumpToRR = Button:create{
		id = "btnGameActionJumpToRockRaider",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		text = "0",
		fSize = 22,
		imageData = Images["icon_rock_raider_frontal"],
		callback = function()
			Game:guiActionJumpToRockRaider()
		end,
		tooltip = {
			text = getString( "game_jump_to_rock_raider_tooltip" ),
		},
		screenId = "game",
	}
	btnJumpToRR:setTextColors( COLORS.white, COLORS.white, COLORS.white, COLORS.white )
	
	----------------------------------------------------------------------------------------------------
	
	-- PRIORITIES (change the order of worker job priorities)
	Button:create{
		id = "btnGameTogglePriorityMenu",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.05,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = getString( "game_priorities" ),
		-- fSize = 22,
		imageData = Images["icon_swap_priorities"],	-- TODO
		callback = function()
			if Game.isPaused then return end
			local isVisible = GUImgr:isVisible( "widGameWorkerJobPriorities" )
			-- hide all other widgets first!
			Game:clearEntitySelection()
			-- clearEntitySelection closes this widget, so this toggle will not work
			-- GUImgr:toggleVisibility( "widGameWorkerJobPriorities" )
			GUImgr:setVisibility( "widGameWorkerJobPriorities", not isVisible )
		end,
		tooltip = {
			text = getString( "game_swap_priority_menu_tooltip" ),
		},
		screenId = "game",
	}
	
	Widget:create{
		id = "widGameWorkerJobPriorities",
		x = SCREEN.width * 0.75,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.3,
		sizeY = SCREEN.height * 0.7,
		screenId = "game",
		visible = false,
	}
	
	do
		local positions = {}
		positions[1] = { x = 0.65, y = 0.2 }
		positions[2] = { x = 0.7, y = 0.25 }
		positions[3] = { x = 0.75, y = 0.3 }
		positions[4] = { x = 0.8, y = 0.35 }
		positions[5] = { x = 0.85, y = 0.4 }
		
		positions[6] = { x = 0.8, y = 0.45 }
		positions[7] = { x = 0.75, y = 0.5 }
		positions[8] = { x = 0.7, y = 0.55 }
		
		positions[9] = { x = 0.65, y = 0.6 }
		positions[10] = { x = 0.7, y = 0.65 }
		positions[11] = { x = 0.75, y = 0.7 }
		positions[12] = { x = 0.8, y = 0.75 }
		positions[13] = { x = 0.85, y = 0.8 }
		
		for index = 1, #positions do
			local button = Button:create{
				id = "btnGameJobPriority"..index,
				x = SCREEN.width * positions[index].x,
				y = SCREEN.height * positions[index].y,
				shape = "rect",
				sizeX = SCREEN.width * 0.08,
				sizeY = SCREEN.height * 0.08,
				autoSizeSquare = true,
				text = "",
				fSize = 22,
				screenId = "game",
				parentId = "widGameWorkerJobPriorities",
				-- enabled = true,
				enabled = index < 11,	-- TODO
				canDrag = true,	-- allow drag and drop for this button
				tooltip = { text = getString( "game_swap_priorities_tooltip" ) },
			}
			button.index = index	-- info which position (index) this button refers to
		end
		Beholder.observe( "EventGuiDragAndDropDone", function(e1, e2) Game:onPriorityButtonDragAndDropDone(e1,e2) end )
	end
	
	-- RESET TO DEFAULT PRIORITIES
	Button:create{
		id = "btnGameResetJobPriorities",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.8,
		sizeX = SCREEN.width * 0.05,
		sizeY = SCREEN.height * 0.05,
		autoSizeSquare = true,
		imageData = Images["icon_settings"],	-- TODO?
		callback = function()
			Game:setDefaultWorkerJobPriorities()
		end,
		tooltip = {
			text = getString( "game_reset_priorities_tooltip" ),
		},
		screenId = "game",
		parentId = "widGameWorkerJobPriorities",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SHORT INFO MESSAGE VIEW
	NotificationView:create{
		id = "novGameShortMessages",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.95,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.1,
		screenId = "game",
		enabled = true,
		-- hideWhenEmpty = true,
		bgColor = COLORS.black,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MINIMAP (RADAR)
	MiniMap:create{
		id = "gameMinimap",
		x = SCREEN.width * 0.3,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.4,
		screenId = "game",
		enabled = true,
		visible = false,
	}
	
	-- OPEN/HIDE MINIMAP
	Button:create{
		id = "btnGameToggleMinimap",
		x = SCREEN.width * 0.05,
		y = SCREEN.height * 0.25,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		tooltip = {
			text = getString( "game_toggle_minimap" ),
		},
		autoSizeSquare = true,
		imageData = Images["icon_minimap"],
		callback = function()
			if Game.isPaused then return end
			GUImgr:toggleVisibility( "gameMinimap" )
		end,
		screenId = "game",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- OPEN PAUSE MENU
	Button:create{
		id = "btnGameOpenPauseMenu",
		x = SCREEN.width * 0.95,
		y = SCREEN.height * 0.95,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		imageData = Images["icon_settings"],
		callback = function()
			Game:setPaused( true )
			GUImgr:setVisibility( "widGamePauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", false )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", false )
		end,
		screenId = "game",
		enabled = true,
		tooltip = { text = getString( "game_open_settings_menu_tooltip" ) },
	}
	
	-- PAUSE MENU
	Widget:create{
		id = "widGamePauseMenu",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.5,
		sizeY = SCREEN.height * 0.6,
		screenId = "game",
		visible = false,
	}
	
	-- CONTINUE GAME
	Button:create{
		id = "btnGameResumeGame",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_resume" ),
		fSize = 22,
		callback = function()
			Game:setPaused( false )
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = true,
		tooltip = { text = getString( "game_resume_game_tooltip" ) },
	}
	
	-- OPTION: CHANGE GAMESPEED
	Label:create{
		id = "lblGameOptionGamespeed",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_gamespeed" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionGamespeedValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionGamespeed",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.3,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0.5,
		maxValue = 2.0,
		value = 1.0,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			GUImgr:getObjectById( "lblGameOptionGamespeedValue" ):setText( string.format( "%.1fx", value ) )
			Game:guiActionSetGamespeed( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	-- OPTION: SOUND VOLUME
	Label:create{
		id = "lblGameOptionSoundVolume",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_sound_volume" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionSoundVolumeValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionSoundVolume",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.4,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0,
		maxValue = 1,
		value = 0.8,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			if value > 0 then
				GUImgr:getObjectById( "lblGameOptionSoundVolumeValue" ):setText( string.format( "%.1f", value ) )
			else
				GUImgr:getObjectById( "lblGameOptionSoundVolumeValue" ):setText( getString( "game_option_off" ) )
			end
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = false,	-- for now! TODO
	}
	
	-- OPTION: MUSIC VOLUME
	Label:create{
		id = "lblGameOptionMusicVolume",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_music_volume" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionMusicVolumeValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionMusicVolume",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0,
		maxValue = 1,
		value = 0.2,
		step = 0.1,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			if value > 0 then
				GUImgr:getObjectById( "lblGameOptionMusicVolumeValue" ):setText( string.format( "%.1f", value ) )
			else
				GUImgr:getObjectById( "lblGameOptionMusicVolumeValue" ):setText( getString( "game_option_off" ) )
			end
			Music:setMasterVolume( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	-- OPTION: SCROLL SPEED (CAMERA PAN)
	Label:create{
		id = "lblGameOptionScrollSpeed",
		x = SCREEN.width * 0.4,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_scroll_speed" ),
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Label:create{
		id = "lblGameOptionScrollSpeedValue",
		x = SCREEN.width * 0.65,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = "-",
		fSize = 22,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	Slider:create{
		id = "sliGameOptionScrollSpeed",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.6,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.05,
		minValue = 0.4,
		maxValue = 2,
		value = 1.0,
		step = 0.2,	-- only allow discrete values
		horizontal = true,
		roundCorners = true,
		onValueChanged = function( value )
			GUImgr:getObjectById( "lblGameOptionScrollSpeedValue" ):setText( string.format( "%.1fx", value ) )
			Game:guiActionSetScrollSpeed( value )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MISSION BRIEFING WIDGET
	Widget:create{
		id = "widGameMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.5,
		sizeY = SCREEN.height * 0.6,
		screenId = "game",
		visible = false,
	}
	
	-- MISSION BRIEFING CAPTION
	Label:create{
		id = "lblGameMissionBriefingCaption",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "game_mission" ),
		fSize = 40,
		screenId = "game",
		parentId = "widGameMissionBriefing",
	}
	
	-- MISSION BRIEFING TEXT
	TextScroll:create{
		id = "txtGameMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.4,
		screenId = "game",
		parentId = "widGameMissionBriefing",
		text = "",
		fSize = 22,
	}
	
	-- OPEN MISSION BRIEFING
	Button:create{
		id = "btnGameOpenMissionBriefing",
		x = SCREEN.width * 0.85,
		y = SCREEN.height * 0.95,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_mission" ),
		fSize = 22,
		callback = function()
			Game:openMissionBriefingWindow()
			-- Game:setPaused( true )
			-- GUImgr:setVisibility( "btnGameOpenMissionBriefing", false )
			-- GUImgr:setVisibility( "btnGameOpenPauseMenu", false )
			-- GUImgr:setVisibility( "widGameMissionBriefing", true )
		end,
		screenId = "game",
		enabled = true,
		tooltip = { text = getString( "game_open_mission_briefing_tooltip" ) },
	}
	
	-- CLOSE MISSION BRIEFING
	Button:create{
		id = "btnGameCloseMissionBriefing",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_resume" ),
		fSize = 22,
		callback = function()
			Game:setPaused( false )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "widGameMissionBriefing", false )
			GUImgr:getObjectById( "btnGameCloseMissionBriefing" ):setText( getString( "game_resume" ) )
		end,
		screenId = "game",
		parentId = "widGameMissionBriefing",
		enabled = true,
		-- this tooltip should not be present here
		-- the button is used on game start and during the game (mission briefing)
		-- tooltip = { text = getString( "game_resume_game_tooltip" ) },
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- CONFIRM QUIT GAME
	Widget:create{
		id = "widGameConfirmQuitGame",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.2,
		screenId = "game",
		visible = false,
	}
	
	-- GOTO TO MAIN MENU CONFIRMATION WIDGET
	Button:create{
		id = "btnGameOpenQuitGameWidget",
		x = SCREEN.width * 0.6,
		y = SCREEN.height * 0.75,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "game_quit" ),
		fSize = 22,
		callback = function()
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "widGameConfirmQuitGame", true )
		end,
		screenId = "game",
		parentId = "widGamePauseMenu",
		enabled = true,
		tooltip = { text = getString( "game_quit_to_main_menu_tooltip" ) },
	}
	
	-- QUIT CONFIRMATION TEXT
	Label:create{
		id = "lblGameConfirmQuitGame",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.45,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString ( "game_confirm_cancel_mission" ),
		fSize = 32,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
	}
	
	-- QUIT TO MAIN MENU
	Button:create{
		id = "btnGameQuit",
		x = SCREEN.width * 0.45,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Ja",
		-- fSize = 22,
		imageData = Images["icon_yes"],
		callback = function()
			GUImgr:setVisibility( "btnGameOpenPauseMenu", true )
			GUImgr:setVisibility( "btnGameOpenMissionBriefing", true )
			GUImgr:setVisibility( "widGamePauseMenu", false )
			GUImgr:setVisibility( "widGameConfirmQuitGame", false )
			Game:close()
			-- MainMenu:open()
			-- PostGame:open( game.gamestate )
		end,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
		enabled = true,
	}
	
	-- DO NOT QUIT TO MAIN MENU
	Button:create{
		id = "btnGameCancelQuitGame",
		x = SCREEN.width * 0.55,
		y = SCREEN.height * 0.55,
		shape = "rect",
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		autoSizeSquare = true,
		-- text = "Nein",
		-- fSize = 22,
		imageData = Images["icon_no"],
		callback = function()
			GUImgr:setVisibility( "widGameConfirmQuitGame", false )
			GUImgr:setVisibility( "widGamePauseMenu", true )
		end,
		screenId = "game",
		parentId = "widGameConfirmQuitGame",
		enabled = true,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- AIR SUPPLY (AIR QUALITY)
	ProgressBar:create{
		id = "prbGameAirQuality",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.105,
		sizeX = SCREEN.width * 0.4,
		sizeY = SCREEN.height * 0.01-2,
		screenId = "game",
		maxValue = 100,
		progress = 100,
		frame = true,
		frameColor = COLORS.lightgray,
		fill = true,
		barColor = COLORS.lightGreen,
		--printText = true,--TODO REMOVE
		--inPercent = true,--TODO REMOVE
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- CURRENT FPS
	Label:create{
		id = "lblGameFps",
		x = SCREEN.width * 0.01,
		y = SCREEN.height * 0.99,
		sizeX = SCREEN.width * 0.02,
		sizeY = SCREEN.height * 0.02,
		text = "",
		-- fSize = 10,
		screenId = "game",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- Mainloop:add( game )
	GUImgr:addScreen( game )
	
	return game
end


return GameScreen
