local Main_Menu = {}
local lg = love.graphics

-- important note on mission names:
-- mission names as defined in the file (displayName) should be unique!
-- multiple missions with the same name could exist, but only one of them can be started!
-- see startSelectedMission

function Main_Menu:create()
	local mm = {}
	mm.id = "main_menu"
	mm.z = 0
	mm.screenId = "main_menu"
	mm.bgColor = { 0.1, 0.1, 0.2 }
	
	-- menu background
	mm.canvas = lg.newCanvas( lg.getWidth(), lg.getHeight() )
	
	
	function mm:open()
		GUImgr:setScreen( self.id )
		
		self:updateMissionList()
		GUImgr:setVisibility( "widMainMenuMissionSelect", false )
		
		Music:switchSetToUse( "menu" )
		
		Logger:log( "Open main menu." )
	end
	
	
	function mm:close()
		Logger:log( "Close main menu." )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function mm:draw()
		if GUImgr:getCurrentScreenId() ~= self.id then
			return
		end
		
		-- TODO: draw the background (mission selection displayed as side-view of caves)
		-- for now just some tests
		lg.setCanvas( self.canvas )
		local imgSize = 128
		lg.setColor( 1, 1, 1 )
		for xi = 1, 3 do
			local img = Images["icon_main_menu_background_"..xi]
			lg.draw( img.image, (xi-0.5) * imgSize, 0.5 * imgSize, 0, 1, 1, 0.5 * imgSize, 0.5 * imgSize )
		end
		lg.setCanvas()
		-- TODO: looks kinda ugly
		-- lg.setBlendMode( "multiply", "premultiplied" )
		-- lg.draw( self.canvas, 0, 0, 0, 4, 4 )
		-- lg.setBlendMode( "alpha" )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function mm:updateMissionList()
		
		-- fetch all missions
		-- get their display names
		self.missions = {}
		local missionNames = {}
		local missionFiles = love.filesystem.getDirectoryItems( "levels/missions" )
		for k, fileNameExt in ipairs( missionFiles ) do
			if string.sub( fileNameExt, -4 ) == ".lua" then
				local fileName = string.sub( fileNameExt, 1, -5 )	-- remove the file extension
				
				local missionData = require( "levels.missions." .. fileName )
				if missionData then
					table.insert( missionNames, missionData.displayName )
					table.insert( self.missions,
						{
							displayName = missionData.displayName,
							fileName = fileName,
							mapName = missionData.mapName,
						}
					)
					-- print( "found mission", missionData.displayName )
				end
			end
		end
		
		self.lslMainMenuMissions:setItems( missionNames )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- important: different missions should have different display names...
	function mm:startSelectedMission()
		
		local missionName = self.lslMainMenuMissions:getSelectedItem()
		if not missionName then
			return false
		end
		
		-- search for the mission (file name)
		for i, m in ipairs( self.missions ) do
			-- see note about displayName (top)
			if m.displayName == missionName then
				-- e.g. "levels.missions.mission2"
				local levelPath = "levels.missions."..m.fileName
				MainMenu:close()
				GameScreen:open()
				-- Game:open()
				Game:startLevel( levelPath )
				return true
			end
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- OPEN TEST MAP
	Button:create{
		id = "btnMainMenuOpenTestMap",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.2,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		-- text = "Test Karte",
		text = getString( "main_menu_test_map" ),
		fSize = 22,
		callback = function()
			print( "start test map" )
			MainMenu:close()
			GameScreen:open()	-- game:open is called from the screen
			-- Game:open()
			Game:startTestMap()
		end,
		screenId = "main_menu",
		-- enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- OPEN CUSTOM MAP CREATOR MENU
	Button:create{
		id = "btnMainMenuOpenCustomMapMenu",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.35,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "main_menu_custom_map_menu" ),
		fSize = 22,
		callback = function()
			MainMenu:close()
			CustomMissionScreen:open()
		end,
		screenId = "main_menu",
		-- enabled = false,
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- TOGGLE MISSION SELECT
	Button:create{
		id = "btnMainMenuToggleMissionSelection",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		-- text = "Soforteinsatz",
		text = getString( "main_menu_new_mission" ),
		fSize = 22,
		callback = function()
			GUImgr:toggleVisibility( "widMainMenuMissionSelect" )
		end,
		screenId = "main_menu",
	}
	
	-- OPEN MAP EDITOR
	Button:create{
		id = "btnMainMenuOpenMapEditor",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.65,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "main_menu_map_editor" ),
		fSize = 22,
		callback = function()
			MainMenu:close()
			MapEditor:open()
		end,
		screenId = "main_menu",
	}
	
	-- OPEN CREDITS
	Button:create{
		id = "btnMainMenuOpenCredits",
		x = SCREEN.width * 0.05,
		y = SCREEN.height * 0.95,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.1,
		text = getString( "main_menu_credits" ),
		fSize = 22,
		callback = function()
			MainMenu:close()
			Credits:open()
		end,
		screenId = "main_menu",
	}
	
	-- QUIT
	Button:create{
		id = "btnMainMenuQuit",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.8,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = getString( "main_menu_quit" ),
		fSize = 22,
		callback = function()
			MainMenu:close()
			Quit()
		end,
		tooltip = {
			text = getString( "main_menu_quit_tooltip" ),
			-- enabled = true,
			duration = -1,
			-- alignMode = "center",
		},
		screenId = "main_menu",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- SELECT MISSION WIDGET
	Widget:create{
		id = "widMainMenuMissionSelect",
		x = SCREEN.width * 0.7,
		y = SCREEN.height * 0.55,
		sizeX = SCREEN.width * 0.1,
		sizeY = SCREEN.height * 0.4,
		screenId = "main_menu",
		visible = false,
	}
	
	-- LIST OF MISSIONS
	mm.lslMainMenuMissions = Listselect:create{
		id = "lslMainMenuMissions",
		screenId = "main_menu",
		parentId = "widMainMenuMissionSelect",
		x = 0.7 * SCREEN.width,
		y = 0.5 * SCREEN.height,
		maxItems = 5,
		sizeX = 0.1 * SCREEN.width-4,
		sizeY = 0.3 * SCREEN.height-4,
		onItemSelected = function( item )
			
			if not item then
				return false
			end
			
			--print( item )
		end,
	}
	
	-- START SELECTED MISSION
	Button:create{
		id = "btnMainMenuStartMission",
		x = SCREEN.width * 0.7,
		y = SCREEN.height * 0.7,
		sizeX = SCREEN.width * 0.09,
		sizeY = SCREEN.height * 0.09,
		text = getString( "main_menu_start_mission" ),
		fSize = 22,
		callback = function()
			mm:startSelectedMission()
		end,
		screenId = "main_menu",
		parentId = "widMainMenuMissionSelect",
	}
	
	----------------------------------------------------------------------------------------------------
	
	Mainloop:add( mm )
	GUImgr:addScreen( mm )
	UIRenderer:add( mm )
	
	return mm
end

return Main_Menu
