local PostGameScreen = {}
local lg = love.graphics

-- see game
local GAMESTATE_PLAYING = 1
local GAMESTATE_WON = 2
local GAMESTATE_FAILED = 3


function PostGameScreen:create()
	local self = {}
	self.id = "post_game"
	self.z = 0
	self.screenId = "post_game"
	
	
	----------------------------------------------------------------------------------------------------
	
	function self:getId()
		return self.id
	end
	
	
	function self:open( gamestate, gametime )
		GUImgr:setScreen( self.id )
		Music:switchSetToUse( "menu" )
		
		local label = GUImgr:getObjectById( "lblPostGameMissionSuccess" )
		if gamestate == GAMESTATE_WON then
			label:setText( getString( "post_game_mission_success" ) )
		elseif gamestate == GAMESTATE_FAILED then
			label:setText( getString( "post_game_mission_failed" ) )
		else
			label:setText( getString( "post_game_mission_canceled" ) )
		end
		
		local minutes = math.floor( gametime / 60 )
		local seconds = gametime - minutes * 60
		-- use %02d to reserve 2 chars and fill them with zeros (%2d would fill with blanks instead)
		--> see: http://www.cplusplus.com/reference/cstdio/printf/
		local timeString = string.format( "%s: %02d:%02d", getString( "post_game_mission_time" ), minutes, seconds )
		GUImgr:getObjectById( "lblPostGameMissionTime" ):setText( timeString )
		
	end
	
	
	function self:close()
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	-- MISSION SUCCESS / FAILURE
	Label:create{
		id = "lblPostGameMissionSuccess",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.25,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = "",
		fSize = 32,
		screenId = "post_game",
	}
	
	-- QUIT TO MAIN MENU
	Button:create{
		id = "btnPostGameQuit",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.95,
		shape = "rect",
		sizeX = SCREEN.width * 0.19,
		sizeY = SCREEN.height * 0.09,
		text = getString( "post_game_quit" ),
		fSize = 22,
		callback = function()
			self:close()
			MainMenu:open()
		end,
		screenId = "post_game",
	}
	
	----------------------------------------------------------------------------------------------------
	
	-- MISSION TIME
	Label:create{
		id = "lblPostGameMissionTime",
		x = SCREEN.width * 0.5,
		y = SCREEN.height * 0.5,
		sizeX = SCREEN.width * 0.2,
		sizeY = SCREEN.height * 0.1,
		text = "",
		fSize = 32,
		screenId = "post_game",
	}
	
	----------------------------------------------------------------------------------------------------
	
	Mainloop:add( self )
	GUImgr:addScreen( self )
	
	return self
end


return PostGameScreen
