local MapPreview = {}
local lg = love.graphics

-- preview of the terrain + walls of a map
-- compared to the minimap this is only static
-- e.g. to display a randomly generated map before the game is started

-- size of each tile in pixel
local tileSize = 8


function MapPreview:create( data )
	local self = {}
	self.id = data.id
	self.z = 2
	self.screenId = data.screenId
	self.canHover = true
	-- self.canClick = true
	
	self.body = {}
	self.body.shape = "rect"
	self.body.sizeX = data.sizeX or 100
	self.body.sizeY = data.sizeY or 100
	self.body.x = data.x or 0
	self.body.y = data.y or 0
	
	self.enabled = data.enabled ~= false
	self.visible = data.visible ~= false
	
	----------------------------------------------------------------------------------------------------
	
	function self:getShape()
		return self.body
	end
	
	----------------------------------------------------------------------------------------------------
	
	function self:clear()
		self.canvas = nil
	end
	
	-- needs sizeX and sizeY, terrain data and wall data
	function self:updateMap( map )
		
		local sizeX = map.sizeX
		local sizeY = map.sizeY
		self.canvasSizeX = sizeX * tileSize
		self.canvasSizeY = sizeY * tileSize
		self.canvas = lg.newCanvas( self.canvasSizeX, self.canvasSizeY )
		
		-- draw terrain and walls on the canvas
		lg.setCanvas( self.canvas )
		lg.clear( 0, 0, 0 )
		
		-- terrain
		for yi = 1, sizeY do
			for xi = 1, sizeX do
				local tileType = map.terrain[yi][xi]
				if tileType == 1 then			-- normal ground
					lg.setColor( COLORS.darkGray )
				elseif tileType == 2 then	-- water
					lg.setColor( COLORS.blue )
				elseif tileType == 3 then	-- lava
					lg.setColor( COLORS.orange )
				end
				if tileType then
					lg.rectangle( "fill", (xi-1)*tileSize, (yi-1)*tileSize, tileSize, tileSize )
				end
			end
		end
		
		-- grid
		lg.setColor( 0, 0, 0 )
		for yi = 1, sizeY do
			for xi = 1, sizeX do
				lg.rectangle( "line", (xi-1)*tileSize, (yi-1)*tileSize, tileSize, tileSize )
			end
		end
		
		-- walls
		for yi = 1, sizeY do
			for xi = 1, sizeX do
				local wallType = map.walls[yi][xi]
				if wallType == ENTITIES.solidRock then
					lg.setColor( COLORS.lightGray )
				elseif wallType == ENTITIES.hardRock then
					lg.setColor( COLORS.brown )
				elseif wallType == ENTITIES.looseRock then
					lg.setColor( COLORS.mediumBrown )
				-- else
					-- lg.setColor( 0, 0, 0, 0 )
				end
				if wallType > 0 then
					lg.rectangle( "fill", (xi-1)*tileSize+1, (yi-1)*tileSize+1, tileSize-2, tileSize-2 )
				end
			end
		end
		
		lg.setCanvas()
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function self:draw()
		if not self.visible then
			return
		end
		
		if GUImgr:getCurrentScreenId() ~= self.screenId then
			return
		end
		
		local body = self.body
		
		-- background
		lg.setColor( 0.1, 0.1, 0.1 )
		lg.rectangle( "fill", body.x - body.sizeX * 0.5, body.y - body.sizeY * 0.5, body.sizeX, body.sizeY )
		
		-- map
		if self.canvas then
			lg.setColor( 1, 1, 1 )
			lg.draw( self.canvas, body.x - body.sizeX * 0.5, body.y - body.sizeY * 0.5 )
		end
		
		-- frame
		lg.setColor( 1, 1, 1, 1 )
		lg.rectangle( "line", body.x - body.sizeX * 0.5, body.y - body.sizeY * 0.5, body.sizeX, body.sizeY )
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	GUImgr:addObject( self )
	UIRenderer:add( self )
	
	return self
end

return MapPreview
