local ProgressBar = {}
local lg = love.graphics

-- simple vertical progress bar

function ProgressBar:create( data )
	local pb = {}
	
	pb.id = data.id
	pb.z = 2
	pb.screenId = data.screenId	-- parent screen (must be!)
	pb.parentId = data.parentId	-- parent widget (optional)
	
	pb.x = data.x
	pb.y = data.y
	pb.sizeX = data.sizeX or SCREEN.width * 0.2
	pb.sizeY = data.sizeY or SCREEN.height * 0.2
	
	pb.enabled = data.enabled ~= false
	pb.visible = data.visible ~= false
	
	pb.progress = data.progress or 0
	
	pb.printText = data.printText	-- should there be a value printed at all?
	pb.alignMode = data.alignMode or "center"
	pb.fontSize = data.fontSize or 24
	pb.textColor = data.textColor or COLORS.white	-- text color
	
	pb.barColor = data.barColor or COLORS.darkBlue
	
	pb.frame = data.frame
	pb.frameColor = data.frameColor or COLORS.darkGray
	
	pb.fill = data.fill
	pb.fillColor = data.fillColor or COLORS.black
	
	pb.inPercent = data.inPercent or false
	pb.minValue = data.minValue or 0
	pb.maxValue = data.maxValue or 100
	
	----------------------------------------------------------------------------------------------------
	
	function pb:setProgress( value )
		if not value or value > self.maxValue or value < self.minValue then
			return false
		end
		
		-- update current progress (absolute value)
		self.progress = value
		
		-- update text
		if self.printText then
			if self.inPercent then
				self:setText( string.format( "%d%%", self:getProgressPercent() ) )
			else
				self:setText( string.format( "%d/%d", self.progress, self.maxValue ) )
			end
		end
	end
	
	
	function pb:setMiniumValue( value )
		self.minValue = value
	end
	
	
	function pb:setMaximumValue( value )
		self.maxValue = value
	end
	
	
	function pb:getProgressPercent()
		local range = self.maxValue - self.minValue
		return self.progress * 100 / range
	end
	
	----------------------------------------------------------------------------------------------------
	
	function pb:setText( txt, alignMode )
		self.text = self.text or lg.newText( fonts:get( self.fontType, self.fontSize ) )
		self.text:setf( txt or "", self.sizeX, alignMode or self.alignMode )
		--print( "progressbar - set font size for", self.id, fSize )
	end
	
	----------------------------------------------------------------------------------------------------
	
	function pb:setBarColor( color )
		self.barColor = color or COLORS.darkBlue
	end
	
	----------------------------------------------------------------------------------------------------
	
	function pb:draw()
		if not self.visible then
			return
		end
		
		if GUImgr:getCurrentScreenId() ~= self.screenId then
			return
		end
		
		if self.parentId and not GUImgr:isVisible(self.parentId) then
			return
		end
		
		-- background
		if self.fill then
			lg.setColor( self.fillColor )
			lg.rectangle( "fill", self.x - self.sizeX * 0.5, self.y - self.sizeY * 0.5, self.sizeX, self.sizeY )
			lg.setColor( COLORS.white )
		end
		
		-- actual progress bar
		lg.setColor( self.barColor )
		local percent = self:getProgressPercent()
		lg.rectangle( "fill", self.x - self.sizeX * 0.5, self.y - self.sizeY * 0.5, self.sizeX * 0.01 * percent, self.sizeY )
		lg.setColor( COLORS.white )
		
		-- outline
		if self.frame then
			lg.setColor( self.frameColor )
			lg.rectangle( "line", self.x - self.sizeX * 0.5, self.y - self.sizeY * 0.5, self.sizeX, self.sizeY )
		end
		
		if self.printText then
			lg.setColor( self.textColor )
			lg.draw( self.text, self.x, self.y, 0, 1, 1, self.sizeX * 0.5, self.text:getHeight() * 0.5 )
			lg.setColor( COLORS.white )
		end
		
	end
	
	----------------------------------------------------------------------------------------------------
	
	function pb:destroy()
		GUImgr:removeObject( self )
		UIRenderer:remove( self )
		self = nil
	end
	
	----------------------------------------------------------------------------------------------------
	
	pb:setProgress( pb.progress )
	
	GUImgr:addObject( pb )
	UIRenderer:add( pb )
	
	return pb
end

return ProgressBar
